package eu.jonahbauer.raytracing.shape;

import eu.jonahbauer.raytracing.math.Range;
import eu.jonahbauer.raytracing.math.Ray;
import eu.jonahbauer.raytracing.render.Color;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.Optional;

public record Scene(@NotNull List<@NotNull Shape> shapes) {

    public Scene {
        shapes = List.copyOf(shapes);
    }

    public Scene(@NotNull Shape @NotNull ... shapes) {
        this(List.of(shapes));
    }

    public @NotNull Optional<HitResult> hit(@NotNull Ray ray) {
        var range = new Range(0, Double.POSITIVE_INFINITY);
        var result = (HitResult) null;
        for (var shape : shapes) {
            var r = shape.hit(ray, range);
            if (r.isPresent() && (result == null || r.get().t() < result.t())) {
                result = r.get();
                range = new Range(0, result.t());
            }
        }
        return Optional.ofNullable(result);
    }

    public @NotNull Color getSkyboxColor(@NotNull Ray ray) {
        // altitude from -pi/2 to pi/2
        var alt = Math.copySign(
                Math.acos(ray.direction().withY(0).unit().times(ray.direction().unit())),
                ray.direction().y()
        );
        return Color.lerp(Color.WHITE, Color.SKY, alt / Math.PI + 0.5);
    }
}
